@@ -1,4 +1,5 @@
-from functions import defun, defun_wrapped
+from ..libmp.backend import xrange
+from .functions import defun, defun_wrapped
 
 def _check_need_perturb(ctx, terms, prec, discard_known_zeros):
     perturb = recompute = False
@@ -77,10 +78,10 @@ def hypercomb(ctx, function, params=[], discard_known_zeros=True, **kwargs):
             params = orig_params[:]
             terms = function(*params)
             if verbose:
-                print
-                print "ENTERING hypercomb main loop"
-                print "prec =", ctx.prec
-                print "hextra", hextra
+                print()
+                print("ENTERING hypercomb main loop")
+                print("prec =", ctx.prec)
+                print("hextra", hextra)
             perturb, recompute, extraprec, discard = \
                 _check_need_perturb(ctx, terms, orig, discard_known_zeros)
             ctx.prec += extraprec
@@ -110,13 +111,13 @@ def hypercomb(ctx, function, params=[], discard_known_zeros=True, **kwargs):
             for term_index, term_data in enumerate(terms):
                 w_s, c_s, alpha_s, beta_s, a_s, b_s, z = term_data
                 if verbose:
-                    print
-                    print "  Evaluating term %i/%i : %iF%i" % \
-                        (term_index+1, len(terms), len(a_s), len(b_s))
-                    print "    powers", ctx.nstr(w_s), ctx.nstr(c_s)
-                    print "    gamma", ctx.nstr(alpha_s), ctx.nstr(beta_s)
-                    print "    hyper", ctx.nstr(a_s), ctx.nstr(b_s)
-                    print "    z", ctx.nstr(z)
+                    print()
+                    print("  Evaluating term %i/%i : %iF%i" % \
+                        (term_index+1, len(terms), len(a_s), len(b_s)))
+                    print("    powers", ctx.nstr(w_s), ctx.nstr(c_s))
+                    print("    gamma", ctx.nstr(alpha_s), ctx.nstr(beta_s))
+                    print("    hyper", ctx.nstr(a_s), ctx.nstr(b_s))
+                    print("    z", ctx.nstr(z))
                 #v = ctx.hyper(a_s, b_s, z, **kwargs)
                 #for a in alpha_s: v *= ctx.gamma(a)
                 #for b in beta_s: v *= ctx.rgamma(b)
@@ -126,7 +127,7 @@ def hypercomb(ctx, function, params=[], discard_known_zeros=True, **kwargs):
                     [ctx.rgamma(b) for b in beta_s] + \
                     [ctx.power(w,c) for (w,c) in zip(w_s,c_s)])
                 if verbose:
-                    print "    Value:", v
+                    print("    Value:", v)
                 evaluated_terms.append(v)
 
             if len(terms) == 1 and (not perturb):
@@ -143,9 +144,9 @@ def hypercomb(ctx, function, params=[], discard_known_zeros=True, **kwargs):
             sum_magnitude = ctx.mag(sumvalue)
             cancellation = max_magnitude - sum_magnitude
             if verbose:
-                print
-                print "  Cancellation:", cancellation, "bits"
-                print "  Increased precision:", ctx.prec - orig, "bits"
+                print()
+                print("  Cancellation:", cancellation, "bits")
+                print("  Increased precision:", ctx.prec - orig, "bits")
 
             precision_ok = cancellation < ctx.prec - orig
 
@@ -184,7 +185,7 @@ def hypercomb(ctx, function, params=[], discard_known_zeros=True, **kwargs):
                 increment = min(max(cancellation, orig//2), max(extraprec,orig))
                 ctx.prec += increment
                 if verbose:
-                    print "  Must start over with increased precision"
+                    print("  Must start over with increased precision")
                 continue
     finally:
         ctx.prec = orig
@@ -198,14 +199,15 @@ def hyper(ctx, a_s, b_s, z, **kwargs):
     z = ctx.convert(z)
     p = len(a_s)
     q = len(b_s)
-    a_s = map(ctx._convert_param, a_s)
-    b_s = map(ctx._convert_param, b_s)
+    a_s = [ctx._convert_param(a) for a in a_s]
+    b_s = [ctx._convert_param(b) for b in b_s]
     # Reduce degree by eliminating common parameters
     if kwargs.get('eliminate', True):
+        elim_nonpositive = kwargs.get('eliminate_all', False)
         i = 0
         while i < q and a_s:
             b = b_s[i]
-            if b in a_s:
+            if b in a_s and (elim_nonpositive or not ctx.isnpint(b[0])):
                 a_s.remove(b)
                 b_s.remove(b)
                 p -= 1
@@ -279,24 +281,22 @@ def _hyp0f1(ctx, b_s, z, **kwargs):
         try:
             # http://functions.wolfram.com/HypergeometricFunctions/
             # Hypergeometric0F1/06/02/03/0004/
-            # We don't need hypercomb because the only possible singularity
-            # occurs when the value is undefined. However, we should perhaps
-            # still check for cancellation...
             # TODO: handle the all-real case more efficiently!
             # TODO: figure out how much precision is needed (exponential growth)
             orig = ctx.prec
             try:
                 ctx.prec += 12 + magz//2
-                w = ctx.sqrt(-z)
-                jw = ctx.j*w
-                u = 1/(4*jw)
-                c = ctx.mpq_1_2 - b
-                E = ctx.exp(2*jw)
-                H1 = (-jw)**c/E*ctx.hyp2f0(b-ctx.mpq_1_2, ctx.mpq_3_2-b, -u,
-                    force_series=True)
-                H2 = (jw)**c*E*ctx.hyp2f0(b-ctx.mpq_1_2, ctx.mpq_3_2-b, u,
-                    force_series=True)
-                v = ctx.gamma(b)/(2*ctx.sqrt(ctx.pi))*(H1 + H2)
+                def h():
+                    w = ctx.sqrt(-z)
+                    jw = ctx.j*w
+                    u = 1/(4*jw)
+                    c = ctx.mpq_1_2 - b
+                    E = ctx.exp(2*jw)
+                    T1 = ([-jw,E], [c,-1], [], [], [b-ctx.mpq_1_2, ctx.mpq_3_2-b], [], -u)
+                    T2 = ([jw,E], [c,1], [], [], [b-ctx.mpq_1_2, ctx.mpq_3_2-b], [], u)
+                    return T1, T2
+                v = ctx.hypercomb(h, [], force_series=True)
+                v = ctx.gamma(b)/(2*ctx.sqrt(ctx.pi))*v
             finally:
                 ctx.prec = orig
             if ctx._is_real_type(b) and ctx._is_real_type(z):
@@ -578,7 +578,7 @@ def _hypq1fq(ctx, p, q, a_s, b_s, z, **kwargs):
             pass
 
         if kwargs.get('verbose'):
-            print "Attempting Euler-Maclaurin summation"
+            print("Attempting Euler-Maclaurin summation")
 
 
         """
@@ -825,9 +825,9 @@ def _hyp1f2(ctx, a_s, b_s, z, **kwargs):
     # Asymptotic series is in terms of 3F0
     can_use_asymptotic = (not kwargs.get('force_series')) and \
         (ctx.mag(absz) > 19) and \
-        (ctx.sqrt(absz) > 1.5*orig) #and \
-        #ctx._hyp_check_convergence([a1, a1-b1+1, a1-b2+1], [],
-        #    1/absz, orig+40+asymp_extraprec)
+        (ctx.sqrt(absz) > 1.5*orig)  # and \
+    #   ctx._hyp_check_convergence([a1, a1-b1+1, a1-b2+1], [],
+    #                              1/absz, orig+40+asymp_extraprec)
 
     # TODO: much of the following could be shared with 2F3 instead of
     # copypasted
@@ -1005,8 +1005,8 @@ def meijerg(ctx, a_s, b_s, z, r=1, series=None, **kwargs):
     q = m + len(bq)
     a = an+ap
     b = bm+bq
-    a = map(ctx.convert, a)
-    b = map(ctx.convert, b)
+    a = [ctx.convert(_) for _ in a]
+    b = [ctx.convert(_) for _ in b]
     z = ctx.convert(z)
     if series is None:
         if p < q: series = 1
@@ -1017,7 +1017,7 @@ def meijerg(ctx, a_s, b_s, z, r=1, series=None, **kwargs):
             else:
                 series = 1
     if kwargs.get('verbose'):
-        print "Meijer G m,n,p,q,series =", m,n,p,q,series
+        print("Meijer G m,n,p,q,series =", m,n,p,q,series)
     if series == 1:
         def h(*args):
             a = args[:p]
@@ -1251,7 +1251,7 @@ def hyper2d(ctx, a, b, x, y, **kwargs):
             args = list(args)
         except TypeError:
             args = [args]
-        return map(ctx.convert, args)
+        return [ctx.convert(arg) for arg in args]
     a_s = dict(a)
     b_s = dict(b)
     a_m = parse(a, 'm')
